@@ -4,6 +4,7 @@ import collections
 import logging
 import re
 import subprocess
+from typing import Any, Dict, Union
 
 from pytype import abstract
 from pytype import abstract_utils
@@ -16,11 +17,13 @@ from pytype import special_builtins
 from pytype import state as frame_state
 from pytype import vm
 from pytype.overlays import typing_overlay
+from pytype.pytd import builtins
+from pytype.pytd import escape
 from pytype.pytd import optimize
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import visitors
-from pytype.pytd.parse import builtins
+from pytype.typegraph import cfg
 
 log = logging.getLogger(__name__)
 
@@ -42,7 +45,8 @@ QUICK_CHECK_MAXIMUM_DEPTH = 2  # during quick checking
 QUICK_INFER_MAXIMUM_DEPTH = 1  # during quick inference
 
 
-_INITIALIZING = object()
+class _Initializing:
+  pass
 
 
 class CallTracer(vm.VirtualMachine):
@@ -56,13 +60,12 @@ class CallTracer(vm.VirtualMachine):
   _CONSTRUCTORS = ("__new__", "__init__")
 
   def __init__(self, *args, **kwargs):
-    super(CallTracer, self).__init__(*args, **kwargs)
+    super().__init__(*args, **kwargs)
     self._unknowns = {}
-    self._builtin_map = {}
     self._calls = set()
     self._method_calls = set()
     # Used by init_class.
-    self._instance_cache = {}
+    self._instance_cache: Dict[Any, Union[_Initializing, cfg.Variable]] = {}
     # Used by call_init. Can differ from _instance_cache because we also call
     # __init__ on classes not initialized via init_class.
     self._initialized_instances = set()
@@ -270,7 +273,7 @@ class CallTracer(vm.VirtualMachine):
     """Set maybe_missing_members to True on these values and their type params.
 
     Args:
-      values: A list of AtomicAbstractValue objects. On every instance among
+      values: A list of BaseValue objects. On every instance among
         the values, recursively set maybe_missing_members to True on the
         instance and its type parameters.
     """
@@ -280,7 +283,7 @@ class CallTracer(vm.VirtualMachine):
       v = values.pop(0)
       if v not in seen:
         seen.add(v)
-        if isinstance(v, abstract.SimpleAbstractValue):
+        if isinstance(v, abstract.SimpleValue):
           v.maybe_missing_members = True
           for child in v.instance_type_parameters.values():
             values.extend(child.data)
@@ -303,8 +306,8 @@ class CallTracer(vm.VirtualMachine):
       A tuple of node and instance variable.
     """
     key = (self.frame and self.frame.current_opcode, extra_key, cls)
-    if (key not in self._instance_cache or
-        self._instance_cache[key] is _INITIALIZING):
+    instance = self._instance_cache.get(key)
+    if not instance or isinstance(instance, _Initializing):
       clsvar = cls.to_variable(node)
       node, instance = self._instantiate_var(node, clsvar)
       if key in self._instance_cache:
@@ -320,10 +323,10 @@ class CallTracer(vm.VirtualMachine):
         # all attribute errors on self in __init__.
         self._mark_maybe_missing_members(instance.data)
       else:
-        self._instance_cache[key] = _INITIALIZING
+        self._instance_cache[key] = _Initializing()
         node = self.call_init(node, instance)
       self._instance_cache[key] = instance
-    return node, self._instance_cache[key]
+    return node, instance
 
   def _call_method(self, node, binding, method_name):
     node, method = self.attribute_handler.get_attribute(
@@ -335,14 +338,15 @@ class CallTracer(vm.VirtualMachine):
     return node
 
   def _call_init_on_binding(self, node, b):
-    if isinstance(b.data, abstract.SimpleAbstractValue):
+    if isinstance(b.data, abstract.SimpleValue):
       for param in b.data.instance_type_parameters.values():
         node = self.call_init(node, param)
     node = self._call_method(node, b, "__init__")
-    # Test classes initialize attributes in setUp() as well.
     cls = b.data.get_class()
-    if isinstance(cls, abstract.InterpreterClass) and cls.is_test_class:
-      node = self._call_method(node, b, "setUp")
+    if isinstance(cls, abstract.InterpreterClass):
+      # Call any additional initalizers the class has registered.
+      for method in cls.additional_init_methods:
+        node = self._call_method(node, b, method)
     return node
 
   def call_init(self, node, instance):
@@ -395,11 +399,11 @@ class CallTracer(vm.VirtualMachine):
             not data.is_overload and
             not data.is_class_builder and
             data.get_first_opcode() not in self._analyzed_functions and
-            not _SKIP_FUNCTION_RE.match(data.name))
+            not _SKIP_FUNCTION_RE.search(data.name))
 
   def analyze_toplevel(self, node, defs):
     for name, var in sorted(defs.items()):  # sort, for determinicity
-      if name not in self._builtin_map:
+      if not self._is_typing_member(name, var):
         for value in var.bindings:
           if isinstance(value.data, abstract.InterpreterClass):
             new_node = self.analyze_class(node, value)
@@ -432,16 +436,6 @@ class CallTracer(vm.VirtualMachine):
     node = node.ConnectNew(name="Analyze")
     return self.analyze_toplevel(node, defs)
 
-  def trace_module_member(self, module, name, member):
-    if module is None or isinstance(module, typing_overlay.TypingOverlay):
-      # TypingOverlay takes precedence over typing.pytd.
-      trace = True
-    else:
-      trace = (module.ast is self.loader.typing
-               and name not in self._builtin_map)
-    if trace:
-      self._builtin_map[name] = member.data
-
   def trace_unknown(self, name, unknown_binding):
     self._unknowns[name] = unknown_binding
 
@@ -496,7 +490,7 @@ class CallTracer(vm.VirtualMachine):
       data.append(pytd.Constant(name, t))
     for name, var in defs.items():
       if (name in output.TOP_LEVEL_IGNORE or name in annotated_names or
-          self._is_builtin(name, var.data)):
+          self._is_typing_member(name, var)):
         continue
       options = var.FilteredData(self.exitpoint, strict=False)
       if (len(options) > 1 and
@@ -552,15 +546,17 @@ class CallTracer(vm.VirtualMachine):
                                      pytd.METHOD))
     return functions
 
-  def _is_builtin(self, name, data):
-    return self._builtin_map.get(name) == data
-
-  def _pack_name(self, name):
-    """Pack a name, for unpacking with type_match.unpack_name_of_partial()."""
-    return "~" + name.replace(".", "~")
+  def _is_typing_member(self, name, var):
+    for module_name in ("typing", "typing_extensions"):
+      if module_name not in self.loaded_overlays:
+        continue
+      module = self.loaded_overlays[module_name].get_module(name)
+      if name in module.members and module.members[name].data == var.data:
+        return True
+    return False
 
   def pytd_functions_for_call_traces(self):
-    return self._call_traces_to_function(self._calls, self._pack_name)
+    return self._call_traces_to_function(self._calls, escape.pack_partial)
 
   def pytd_classes_for_call_traces(self):
     class_to_records = collections.defaultdict(list)
@@ -577,12 +573,13 @@ class CallTracer(vm.VirtualMachine):
     for cls, call_records in class_to_records.items():
       full_name = cls.module + "." + cls.name if cls.module else cls.name
       classes.append(pytd.Class(
-          name=self._pack_name(full_name),
+          name=escape.pack_partial(full_name),
           metaclass=None,
-          parents=(pytd.NamedType("__builtin__.object"),),  # not used in solver
+          parents=(pytd.NamedType("builtins.object"),),  # not used in solver
           methods=tuple(self._call_traces_to_function(call_records)),
           constants=(),
           classes=(),
+          decorators=(),
           slots=None,
           template=(),
       ))
@@ -604,7 +601,7 @@ class CallTracer(vm.VirtualMachine):
     ty = ty.Visit(optimize.CombineReturnsAndExceptions())
     ty = ty.Visit(optimize.PullInMethodClasses())
     ty = ty.Visit(visitors.DefaceUnresolved(
-        [ty, self.loader.concat_all()], "~unknown"))
+        [ty, self.loader.concat_all()], escape.UNKNOWN))
     return ty.Visit(visitors.AdjustTypeParameters())
 
   def _check_return(self, node, actual, formal):
@@ -618,11 +615,8 @@ class CallTracer(vm.VirtualMachine):
     if not bad:
       bad = self.matcher.bad_matches(actual, formal, node)
     if bad:
-      with self.convert.pytd_convert.produce_detailed_output():
-        combined = pytd_utils.JoinTypes(
-            view[actual].data.to_type(node, view=view) for view in bad)
-        self.errorlog.bad_return_type(
-            self.frames, combined, formal.get_instance_type(node))
+      self.errorlog.bad_return_type(
+          self.frames, node, formal, actual, bad)
     return not bad
 
 
@@ -741,5 +735,5 @@ def _maybe_output_debug(options, program):
     if options.output_debug == "-":
       log.info("=========== Program Dump =============\n%s", text)
     else:
-      with open(options.output_debug, "w") as fi:
+      with options.open_function(options.output_debug, "w") as fi:
         fi.write(text)
