@@ -1,5 +1,3 @@
-# Lint as: python3
-
 """Classes for instrumenting code to collect various metrics.
 
 Instrumentation consists of creating the metric and then updating it.
@@ -25,17 +23,10 @@ import math
 import re
 import sys
 import time
+import tracemalloc
 
 import six
 
-
-# TODO(tsudol): Not needed once pytype is ported to Python 3.
-try:
-  import tracemalloc  # pytype: disable=import-error  # pylint: disable=g-import-not-at-top
-except ImportError:
-  tracemalloc = None
-
-
 # Metric serialization/deserialization code, taking advantage of the fact that
 # all instance vars of metrics are things that json can serialize, so we don't
 # need to write custom JsonEncoder and JsonDecoder classes per Metric subclass.
@@ -99,10 +90,8 @@ def _prepare_for_test(enabled=True):
 
 def get_cpu_clock():
   """Returns CPU clock to keep compatibilty with various Python versions."""
-  # https://github.com/google/pytype/issues/473: pytype doesn't recognize
-  # version_info tuple comparisons.
   if sys.version_info >= (3, 3):
-    return time.process_time()  # pytype: disable=module-attr
+    return time.process_time()
 
   # time.clock() is deprecated since Python 3.3 and removed in Python 3.8.
   return time.clock()
@@ -149,7 +138,7 @@ def merge_from_file(metrics_file):
 
 
 @six.add_metaclass(_RegistryMeta)
-class Metric(object):
+class Metric:
   """Abstract base class for metrics."""
 
   def __init__(self, name):
@@ -185,7 +174,7 @@ class Counter(Metric):
   """A monotonically increasing metric."""
 
   def __init__(self, name):
-    super(Counter, self).__init__(name)
+    super().__init__(name)
     self._total = 0
 
   def inc(self, count=1):
@@ -226,7 +215,7 @@ class ReentrantStopWatch(Metric):
   """A watch that supports being called multiple times and recursively."""
 
   def __init__(self, name):
-    super(ReentrantStopWatch, self).__init__(name)
+    super().__init__(name)
     self._time = 0
     self._calls = 0
 
@@ -252,7 +241,7 @@ class MapCounter(Metric):
   """A set of related counters keyed by an arbitrary string."""
 
   def __init__(self, name):
-    super(MapCounter, self).__init__(name)
+    super().__init__(name)
     self._counts = {}
     self._total = 0
 
@@ -289,7 +278,7 @@ class Distribution(Metric):
   """A metric to track simple statistics from a distribution of values."""
 
   def __init__(self, name):
-    super(Distribution, self).__init__(name)
+    super().__init__(name)
     self._count = 0  # Number of values.
     self._total = 0.0  # Sum of the values.
     self._squared = 0.0  # Sum of the squares of the values.
@@ -350,7 +339,7 @@ class Snapshot(Metric):
 
   def __init__(self, name, enabled=False, groupby="lineno",
                nframes=1, count=10):
-    super(Snapshot, self).__init__(name)
+    super().__init__(name)
     self.snapshots = []
     # The metric to group memory blocks by. Default is "lineno", which groups by
     # which file and line allocated the block. The other useful value is
@@ -362,12 +351,11 @@ class Snapshot(Metric):
     # The number of memory block statistics to save.
     self.count = count
     self.running = False
-    # Three conditions must be met for memory snapshots to be taken:
-    # 1. tracemalloc was imported successfully (tracemalloc)
-    # 2. Metrics have been enabled (global _enabled)
-    # 3. Explicitly enabled by the arg to the constructor (which should be the
+    # Two conditions must be met for memory snapshots to be taken:
+    # 1. Metrics have been enabled (global _enabled)
+    # 2. Explicitly enabled by the arg to the constructor (which should be the
     # options.memory_snapshot flag set by the --memory-snapshots option)
-    self.enabled = tracemalloc and _enabled and enabled
+    self.enabled = _enabled and enabled
 
   def _start_tracemalloc(self):
     tracemalloc.start(self.nframes)
@@ -408,17 +396,19 @@ class Snapshot(Metric):
     return "\n\n".join(self.snapshots)
 
 
-class MetricsContext(object):
+class MetricsContext:
   """A context manager that configures metrics and writes their output."""
 
-  def __init__(self, output_path):
+  def __init__(self, output_path, open_function=open):
     """Initialize.
 
     Args:
       output_path: The path for the metrics data.  If empty, no metrics are
           collected.
+      open_function: A custom file opening function.
     """
     self._output_path = output_path
+    self._open_function = open_function
     self._old_enabled = None  # Set in __enter__.
 
   def __enter__(self):
@@ -430,5 +420,5 @@ class MetricsContext(object):
     global _enabled
     _enabled = self._old_enabled
     if self._output_path:
-      with open(self._output_path, "w") as f:
+      with self._open_function(self._output_path, "w") as f:
         dump_all(_registered_metrics.values(), f)
