@@ -14,15 +14,15 @@ class ABCOverlay(overlay.Overlay):
         "abstractproperty": AbstractProperty
     }
     ast = vm.loader.import_name("abc")
-    super(ABCOverlay, self).__init__(vm, "abc", member_map, ast)
+    super().__init__(vm, "abc", member_map, ast)
 
 
 class AbstractMethod(abstract.PyTDFunction):
   """Implements the @abc.abstractmethod decorator."""
 
   @classmethod
-  def make(cls, name, vm):
-    return super(AbstractMethod, cls).make(name, vm, "abc")
+  def make(cls, vm):
+    return super().make("abstractmethod", vm, "abc")
 
   def call(self, node, unused_func, args):
     """Marks that the given function is abstract."""
@@ -44,8 +44,8 @@ class AbstractMethod(abstract.PyTDFunction):
 class AbstractProperty(special_builtins.PropertyTemplate):
   """Implements the @abc.abstractproperty decorator."""
 
-  def __init__(self, name, vm):
-    super(AbstractProperty, self).__init__(vm, name, "abc")
+  def __init__(self, vm):
+    super().__init__(vm, "abstractproperty", "abc")
 
   def call(self, node, funcv, args):
     property_args = self._get_args(args)
