@@ -27,15 +27,15 @@ class DataclassOverlay(overlay.Overlay):
         "field": Field.make,
     }
     ast = vm.loader.import_name("dataclasses")
-    super(DataclassOverlay, self).__init__(vm, "dataclasses", member_map, ast)
+    super().__init__(vm, "dataclasses", member_map, ast)
 
 
 class Dataclass(classgen.Decorator):
   """Implements the @dataclass decorator."""
 
   @classmethod
-  def make(cls, name, vm, mod="dataclasses"):
-    return super(Dataclass, cls).make(name, vm, mod)
+  def make(cls, vm, mod="dataclasses"):
+    return super().make("dataclass", vm, mod)
 
   def _handle_initvar(self, node, cls, name, typ, orig):
     """Unpack or delete an initvar in the class annotations."""
@@ -49,6 +49,12 @@ class Dataclass(classgen.Decorator):
       cls.members[name] = classgen.instantiate(node, name, initvar)
     return initvar
 
+  def get_class_locals(self, node, cls):
+    del node
+    return classgen.get_class_locals(
+        cls.name, allow_methods=True, ordering=classgen.Ordering.FIRST_ANNOTATE,
+        vm=self.vm)
+
   def decorate(self, node, cls):
     """Processes class members."""
 
@@ -60,9 +66,7 @@ class Dataclass(classgen.Decorator):
     #   x = 10
     # would have init(x:int = 10, y:str = 'hello')
     own_attrs = []
-    cls_locals = classgen.get_class_locals(
-        cls.name, allow_methods=True, ordering=classgen.Ordering.FIRST_ANNOTATE,
-        vm=self.vm)
+    cls_locals = self.get_class_locals(node, cls)
     for name, local in cls_locals.items():
       typ, orig = local.get_type(node, name), local.orig
       assert typ
@@ -85,6 +89,7 @@ class Dataclass(classgen.Decorator):
       # TODO(b/74434237): The first check can be removed once
       # --check-variable-types is on by default.
       if ((not self.vm.options.check_variable_types and
+           local.last_op and
            local.last_op.line in self.vm.director.type_comments) or
           orig and orig.data == [self.vm.convert.none]):
         # vm._apply_annotation mostly takes care of checking that the default
@@ -110,11 +115,11 @@ class Dataclass(classgen.Decorator):
       cls.members["__init__"] = init_method
 
 
-class FieldInstance(abstract.SimpleAbstractValue):
+class FieldInstance(abstract.SimpleValue):
   """Return value of a field() call."""
 
   def __init__(self, vm, init, default):
-    super(FieldInstance, self).__init__("field", vm)
+    super().__init__("field", vm)
     self.init = init
     self.default = default
     self.cls = vm.convert.unsolvable
@@ -124,8 +129,8 @@ class Field(classgen.FieldConstructor):
   """Implements dataclasses.field."""
 
   @classmethod
-  def make(cls, name, vm):
-    return super(Field, cls).make(name, vm, "dataclasses")
+  def make(cls, vm):
+    return super().make("field", vm, "dataclasses")
 
   def call(self, node, unused_func, args):
     """Returns a type corresponding to a field."""
