@@ -1,4 +1,3 @@
-#!/usr/bin/python2.7
 """Tool for processing pytd files.
 
 pytd is a type declaration language for Python. Each .py file can have an
@@ -15,9 +14,9 @@ import sys
 
 from pytype import utils
 from pytype.pyi import parser
+from pytype.pytd import builtins
 from pytype.pytd import optimize
 from pytype.pytd import pytd_utils
-from pytype.pytd.parse import builtins
 
 
 def make_parser():
