@@ -79,7 +79,7 @@ class Constant(node.Node('name: str', 'type: {Type}')):
   __slots__ = ()
 
 
-class Alias(node.Node('name: str', 'type: {Type} or Constant')):
+class Alias(node.Node('name: str', 'type: {Type} or Constant or Function')):
   """An alias (symbolic link) for a class implemented in some other module.
 
   Unlike Constant, the Alias is the same type, as opposed to an instance of that
@@ -223,6 +223,10 @@ class Signature(node.Node('params: tuple[Parameter]',
   """
   __slots__ = ()
 
+  @property
+  def name(self):
+    return None
+
   @property
   def has_optional(self):
     return self.starargs is not None or self.starstarargs is not None
@@ -394,36 +398,17 @@ class LateType(node.Node('name: str'), Type):
     return self.name
 
 
-class FunctionType(node.Node('name: str'), Type):
-  """The type of a function. E.g. the type of 'x' in 'x = lambda y: y'."""
-
-  def __new__(cls, name, function=None):
-    self = super(FunctionType, cls).__new__(cls, name)
-    self.function = function
-    return self
-
-  def __getstate__(self):
-    return (self.function,)
-
-  def __setstate__(self, state):
-    self.function = state[0]
-
-  def __repr__(self):
-    return '{type}{cls}({name})'.format(
-        type=type(self).__name__, name=self.name,
-        cls='<unresolved>' if self.function is None else '')
-
-
 class AnythingType(node.Node(), Type):
   """A type we know nothing about yet ('?' in pytd)."""
   __slots__ = ()
 
+  @property
+  def name(self):
+    return None
+
   def __bool__(self):
     return True
 
-  # For running under Python 2
-  __nonzero__ = __bool__
-
 
 class NothingType(node.Node(), Type):
   """An "impossible" type, with no instances ('nothing' in pytd).
@@ -433,12 +418,13 @@ class NothingType(node.Node(), Type):
   """
   __slots__ = ()
 
+  @property
+  def name(self):
+    return None
+
   def __bool__(self):
     return True
 
-  # For running under Python 2
-  __nonzero__ = __bool__
-
 
 class _SetOfTypes(node.Node('type_list: tuple[{Type}]'), Type):
   """Super class for shared behavior of UnionType and IntersectionType."""
@@ -460,6 +446,10 @@ class _SetOfTypes(node.Node('type_list: tuple[{Type}]'), Type):
 
     return super(_SetOfTypes, cls).__new__(cls, unique)
 
+  @property
+  def name(self):
+    return None
+
   def __hash__(self):
     # See __eq__ - order doesn't matter, so use frozenset
     return hash(frozenset(self.type_list))
@@ -496,6 +486,10 @@ class GenericType(node.Node('base_type: NamedType or ClassType or LateType',
   """
   __slots__ = ()
 
+  @property
+  def name(self):
+    return self.base_type.name
+
   @property
   def element_type(self):
     """Type of the contained type, assuming we only have one type parameter."""
@@ -533,6 +527,10 @@ class CallableType(GenericType):
 class Literal(node.Node('value: int or str or {Type}'), Type):
   __slots__ = ()
 
+  @property
+  def name(self):
+    return None
+
 
 # Types that can be a base type of GenericType:
 GENERIC_BASE_TYPE = (NamedType, ClassType)
@@ -550,27 +548,51 @@ def IsContainer(t):
   return False
 
 
-def ToType(item, allow_constants=True):
-  """Convert a pytd AST item into a type."""
+def ToType(item, allow_constants=False, allow_functions=False):
+  """Convert a pytd AST item into a type.
+
+  Takes an AST item representing the definition of a type and returns an item
+  representing a reference to the type. For example, if the item is a
+  pytd.Class, this method will return a pytd.ClassType whose cls attribute
+  points to the class.
+
+  Args:
+    item: A pytd.Node item.
+    allow_constants: When True, constants that cannot be converted to types will
+      be passed through unchanged.
+    allow_functions: When True, functions that cannot be converted to types will
+      be passed through unchanged.
+
+  Returns:
+    A pytd.Type object representing the type of an instance of `item`.
+  """
   if isinstance(item, Type):
     return item
   elif isinstance(item, Module):
     return item
   elif isinstance(item, Class):
     return ClassType(item.name, item)
-  elif isinstance(item, Function):
-    return FunctionType(item.name, item)
-  elif isinstance(item, Constant):
-    if allow_constants:
-      # TODO(b/159053187): This is wrong. It would be better if we resolve Alias
-      # in the same way we resolve NamedType.
-      return item
+  elif isinstance(item, Function) and allow_functions:
+    return item
+  elif (isinstance(item, Constant) and item.type.name == 'builtins.type'):
+    # A constant whose type is Type[C] is equivalent to class C, so the type of
+    # an instance of the constant is C.
+    if isinstance(item.type, GenericType):
+      return item.type.parameters[0]
     else:
-      # TODO(b/159053187): We should be more picky here. In particular, we
-      # shouldn't allow pyi like this:
-      #  object = ...  # type: int
-      #  def f(x: object) -> Any
       return AnythingType()
+  elif isinstance(item, Constant) and (
+      isinstance(item.type, AnythingType) or
+      item.type.name == 'typing_extensions._SpecialForm' or
+      item.name == 'typing_extensions.TypedDict'):
+    # The following constants are treated as (unknown) types:
+    # * one whose type is Any or typing_extensions._SpecialForm
+    # * one whose name is typing_extensions.TypedDict, as TypedDict is a class
+    #   that looks like a constant:
+    #   https://github.com/python/typeshed/blob/8cad322a8ccf4b104cafbac2c798413edaa4f327/third_party/2and3/typing_extensions.pyi#L68
+    return AnythingType()
+  elif isinstance(item, Constant) and allow_constants:
+    return item
   elif isinstance(item, Alias):
     return item.type
   else:
